package duc.util;

import java.text.*;
import java.io.*;

public class CollationFactory {
	private static Collator fInstance;
	static {
		fInstance = createCollator(TCVN1Encoding.getInstance());
	}
	private static Collator createCollator(Encoding e) {
		try {
			return new RuleBasedCollator(getCollationString(e));
		} catch (ParseException pe) { return Collator.getInstance(); }
	}
private static String getCollationString(Encoding e) {
	StringBuffer sb = new StringBuffer(180);
	sb.append("<' '='-'");
	sb.append("<a");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('a', i));
	}
	sb.append(",A");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('A', i));
	}
	char a_breve = e.addBreveHorn('a');
	sb.append("<");
	sb.append(a_breve);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(a_breve, i));
	}
	char cap_a_breve = e.addBreveHorn('A');
	sb.append(",");
	sb.append(cap_a_breve);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(cap_a_breve, i));
	}
	char a_circ = e.addCircumflex('a');
	sb.append("<");
	sb.append(a_circ);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(a_circ, i));
	}
	char cap_a_circ = e.addCircumflex('A');
	sb.append(",");
	sb.append(cap_a_circ);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(cap_a_circ, i));
	}
	sb.append("<b,B<c,C<d,D");
	sb.append("<");
	sb.append(e.addStroke('d'));
	sb.append(",");
	sb.append(e.addStroke('D'));
	sb.append("<e");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('e', i));
	}
	sb.append(",E");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('E', i));
	}
	char e_circ = e.addCircumflex('e');
	sb.append("<");
	sb.append(e_circ);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(e_circ, i));
	}
	char cap_e_circ = e.addCircumflex('E');
	sb.append(",");
	sb.append(cap_e_circ);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(cap_e_circ, i));
	}
	sb.append("<f,F<g,G<h,H");
	sb.append("<i");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('i', i));
	}
	sb.append(",I");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('I', i));
	}
	sb.append("<j,J<k,K<l,L<m,M<n,N");
	sb.append("<o");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('o', i));
	}
	sb.append(",O");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('O', i));
	}
	char o_circ = e.addCircumflex('o');
	sb.append("<");
	sb.append(o_circ);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(o_circ, i));
	}
	char cap_o_circ = e.addCircumflex('O');
	sb.append(",");
	sb.append(cap_o_circ);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(cap_o_circ, i));
	}
	char o_horn = e.addBreveHorn('o');
	sb.append("<");
	sb.append(o_horn);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(o_horn, i));
	}
	char cap_o_horn = e.addBreveHorn('O');
	sb.append(",");
	sb.append(cap_o_horn);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(cap_o_horn, i));
	}
	sb.append("<p,P<q,Q<r,R<s,S<t,T");
	sb.append("<u");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('u', i));
	}
	sb.append(",U");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('U', i));
	}
	char u_horn = e.addBreveHorn('u');
	sb.append("<");
	sb.append(u_horn);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(u_horn, i));
	}
	char cap_u_horn = e.addBreveHorn('U');
	sb.append(",");
	sb.append(cap_u_horn);
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone(cap_u_horn, i));
	}
	sb.append("<v,V<w,W<x,X");
	sb.append("<y");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('y', i));
	}
	sb.append(",Y");
	for (int i = 1; i < 6; i++) {
		sb.append(";");
		sb.append(e.modifyTone('Y', i));
	}
	sb.append("<z,Z");
	return sb.toString();
}
/**
 * Return the default collator for Vietnamese texts, based on TCVN1 encoding.
 * Texts in other encodings can be compared by converting to TCVN1 first.
 */
public static Collator getCollator() {
	return fInstance;
}
}
