package duc.util;

import java.text.*;
import java.util.*;
/**
 * Could be used as follows:
 * <pre>
 * String[] arr = {"ca?nh", "ca'", "ca`n", "ca`", "ca`nh", "ca?n", "ca."};
 * VietTextUtil.sortVietStrings(VIQREncoding.getInstance(), arr);
 * </pre>
 * Creation date: (3/26/00 10:23:17 PM)
 * @author: Administrator
 */
public class VietTextUtil {
/**
 * VietTextUtil constructor comment.
 */
public VietTextUtil() {
	super();
}
private static int compare(java.text.CollationKey[] arr1, java.text.CollationKey[] arr2) {
	int i, k;
	for (i = 0; i < Math.min(arr1.length, arr2.length); i++) {
		k = arr1[i].compareTo(arr2[i]);
		if (k != 0)
			return k;
	}
	return arr1.length - arr2.length;
}
private static int compare(java.text.Collator c, String[] arr1, String[] arr2) {
	int i, k;
	for (i = 0; i < Math.min(arr1.length, arr2.length); i++) {
		k = c.compare(arr1[i], arr2[i]);
		if (k != 0)
			return k;
	}
	return arr1.length - arr2.length;
}
public static int compare(java.text.Collator c, String s1, String s2) {
	return compare(c, getTokens(s1), getTokens(s2));
}
	private static CollationKey[] getCollationKeys(Collator c, String s1) {
	String[] arr1 = getTokens(s1, "- ", true);
	CollationKey[] ret = new CollationKey[arr1.length];
	for (int i=0; i<ret.length; i++) {
	    ret[i] = c.getCollationKey(arr1[i]);
	}
	return ret;
	}
private static String[] getTokens(String s1) {
	return getTokens(s1, " -", true);
}
private static String[] getTokens(String s1, String delim, boolean flag) {
	StringTokenizer st1 = new StringTokenizer(s1, delim, flag);
	Vector v = new Vector();
	while (st1.hasMoreTokens()) {
		v.addElement(st1.nextToken());
	}
	String[] arr = new String[v.size()];
	v.copyInto(arr);
	return arr;
}
/**
 * Sort a list of Strings, assumed that they are in TCVN1 encoding
 * Creation date: (3/26/00 10:38:57 PM)
 */
public static void sort(String[] list) {
	Collator c = CollationFactory.getCollator();
	sort(c, list);
}
/**
 * Sort a list of Strings encoding in a certain encoding.
 * Whole strings are compared.
 */
public static void sort(Encoding e, String[] list) {
	String[] arr = new String[list.length];
	for (int i = 0; i < arr.length; i++){
		arr[i] = e.toTCVN1(list[i]);
	}
	sort(arr);
	for (int i = 0; i < arr.length; i++){
		list[i] = e.fromTCVN1(arr[i]);
	}
}
	private static void sort(Collator c, String[] list) {
	CollationKey k1, k2;
	java.util.Vector v = new java.util.Vector(list.length);
	for (int i=0; i<list.length; i++) {
	    v.addElement(c.getCollationKey(list[i]));
	}
	for (int i=1; i<v.size(); i++) {
	    k1 = (CollationKey)v.elementAt(i);
	    for (int k=0; k<i; k++) {
		k2 = (CollationKey)v.elementAt(k);
		if (k1.compareTo(k2) < 0) {
		    v.removeElementAt(i);
		    v.insertElementAt(k1,k);
		    break;
		}
	    }
	}
	for (int i=0; i<list.length; i++) {
	    list[i] = ((CollationKey)v.elementAt(i)).getSourceString();
	}
	}
/**
 * Sort a list of Strings, assumed that they are in TCVN1 encoding.
 * The strings are compared wordwise, i.e., they are decomposed into words 
 * -- separated by ' ' (blank) or - (minus sign) before comparison.
 * Creation date: (3/26/00 10:38:57 PM)
 */
public static void sortVietStrings(String[] list) {
	Collator c = CollationFactory.getCollator();
	sortVietStrings(c, list);
}
/**
 * Sort a list of Strings encoding in a certain encoding.
 * The strings are compared wordwise, i.e., they are decomposed into words 
 * -- separated by ' ' (blank) or '-' (minus sign) -- before comparison.
 * Creation date: (3/26/00 10:38:57 PM)
 */
public static void sortVietStrings(Encoding e, String[] list) {
	String[] arr = new String[list.length];
	for (int i = 0; i < arr.length; i++){
		arr[i] = e.toTCVN1(list[i]);
	}
	sortVietStrings(arr);
	for (int i = 0; i < arr.length; i++){
		list[i] = e.fromTCVN1(arr[i]);
	}
}
private static void sortVietStrings(Collator c, String[] list) {
	CollationKey[] k1, k2;
	StringBuffer sb;
	java.util.Vector v = new java.util.Vector(list.length);
	for (int i = 0; i < list.length; i++) {
		v.addElement(getCollationKeys(c, list[i]));
	}
	for (int i = 1; i < v.size(); i++) {
		k1 = (CollationKey[]) v.elementAt(i);
		for (int k = 0; k < i; k++) {
			k2 = (CollationKey[]) v.elementAt(k);
			if (compare(k1, k2) < 0) {
				v.removeElementAt(i);
				v.insertElementAt(k1, k);
				break;
			}
		}
	}
	for (int i = 0; i < list.length; i++) {
		k1 = (CollationKey[]) v.elementAt(i);
		sb = new StringBuffer();
		for (int k = 0; k < k1.length; k++) {
			sb.append(k1[k].getSourceString());
			//if (k!=k1.length-1) sb.append(' ');
		}
		list[i] = sb.toString();
	}
}
	public static String toLowerCase(Encoding c, String str) {
		char[] data = new char[str.length()];
		for (int i = 0; i < data.length; i++){
			data[i] = c.toLowerCase((char)(str.charAt(i)&0xff));
		}
		return new String(data);
	}
}
