package vietdict.common;

import java.util.*;

public class Entry
{

	private String word, pronounciation, alternatives;
	private String html7, html8, text7, text8;
	String neighbors[] = new String[0];
	Vector wordKinds = new Vector();
	public static String BEGIN = "@";

	public void addWordKind(WordKind aHeadWord) 
	{
		getWordKinds().addElement(aHeadWord);
	}
	public String getAlternatives() 
	{
		return alternatives;
	}
	public String[] getNeighbors()
	{
		return neighbors;
	}
	public String getPronounciation() 
	{
		return pronounciation;
	}
	public String getWord() 
	{
		return word;
	}
	public Vector getWordKinds() 
	{
		return wordKinds;
	}
/**
 * Parse a string to fill a dictionary entry
 * aLine is a String of the form: @word [words] /pron/ [others]
 */

public void parse(String aLine) {
    //System.out.println(aLine);
	int i, k;
	i = aLine.indexOf('/');
	k = aLine.indexOf('/', i + 1);
	if ((i >= 0) && (k >= 0)) {
		setWord(aLine.trim().substring(1, i));
		setPronounciation(aLine.substring(i + 1, k));
		setAlternatives(aLine.substring(k + 1).trim());
	} else {
		setWord(aLine.trim().substring(1));
	}
}
	public void setAlternatives(String aString) 
	{
		alternatives = aString;
	}
	public void setNeighbors(String[] arr)
	{
		neighbors = arr;
	}
	public void setPronounciation(String aString) 
	{
		pronounciation = aString;
	}
	public void setWord(String aWord) 
	{
		word = aWord; 
	}
	public void setWordKinds(Vector aVector) 
	{
		wordKinds = aVector;
	}

    protected String convert(String s) {
	return s;
    }

    protected void printFontTag(StringBuffer sb) {
	sb.append("<FONT FACE=\".VnTime\">");
    }

public String toHtml(int encoding, String base) {
	if (html8 == null) {
	    StringBuffer sb = new StringBuffer();
	    printFontTag(sb);
	    sb.append("\n<H2> ");
	    sb.append(convert(getWord()));
	    sb.append(" </H2>");
	    sb.append("\n</FONT>\n<FONT FACE=\".VnTime\">");
	    if (getPronounciation() != null) {
			sb.append("\n/");
			sb.append(VSDictEntry.toVIQR(getPronounciation()));
			sb.append("/");
		}
		if (getAlternatives() != null) {
			sb.append(" ");
			sb.append(VSDictEntry.toVIQR(getAlternatives()));
		}
		for (int i = 0; i < wordKinds.size(); i++) {
			sb.append(((WordKind) wordKinds.elementAt(i)).toHtml(base));
		}
		sb.append("\n</FONT>\n");
		printFontTag(sb);
		if (getNeighbors().length > 0) {
			sb.append("\n<HR>\n");
			for (int i = 0; i < getNeighbors().length; i++) {
			    Parser.insertLink(convert(getNeighbors()[i]), base, sb);
			    sb.append(" |\n");
			}
		}
		sb.append("\n</FONT>");
		html8 = sb.toString();
	}
	if (encoding == 8) {
		return html8;
	} else {
		html7 = toVIQR(html8);
		return html7;
	}
}
	public String toString() 
	{
		return toString(8);
	}
	/**
	 * Print the receiver as a 7-bit or 8-bit encoded String
	 * @encoding int Valid values are 7 (VIQR) and 8 (TCVN)
	 */

	public String toString(int encoding) 
	{
		if (text8 == null) 
		{
			StringBuffer sb = new StringBuffer();
			sb.append("@");
			sb.append(getWord());
			if (getPronounciation() != null) {
			sb.append(" /");
			sb.append(VSDictEntry.toVIQR(getPronounciation()));
			sb.append("/");
			}
			if (getAlternatives() != null) 
			{
				sb.append(" ");
				sb.append(getAlternatives());
			}
			for (int i=0; i<wordKinds.size(); i++) 
			{
				sb.append(((WordKind)wordKinds.elementAt(i)).toString());
			}
			if (getNeighbors().length > 0) 
			{
				sb.append("\nNearby entries:\n");
				for(int i = 0; i < getNeighbors().length; i++)
				{
					sb.append(getNeighbors()[i]);
					sb.append(" | ");
				}
			}
			text8 = sb.toString();
		}
		if (encoding == 8)
		{
			return text8;
		}
		else
		{
			text7 = toVIQR(text8);
			return text7;
		}
	}
	public String toString(boolean tcvn, boolean html, String base) 
	{
		int encoding = 7;
		if (tcvn) encoding = 8;
		if (html)
		{
			return toHtml(encoding, base);
		}
		else
		{
			return toString(encoding);
		}
	}
	protected String toVIQR(String tcvn)
	{
		return VSDictEntry.toVIQR(tcvn);
	}
}
