package vietdict.common;

import java.io.*;
import java.util.*;

public class Parser 
{

	public static String LINK = "$";
	public static String BASE = "/td?word=";

	DataInput in;

public Parser() {
	super();
}
public Entry createEntry() {
	return new Entry();
}
public Example createExample(String s1, String s2) {
	return new Example(s1, s2);
}
public Idiom createIdiom(String s) {
	return new Idiom(s);
}
public Meaning createMeaning(String s) {
	return new Meaning(s);
}
public WordKind createWordKind() {
	return new WordKind();
}
public static void insertLink(String word, String base, StringBuffer sb) {
	sb.append("<A HREF=\"");
	sb.append(base);
	sb.append(java.net.URLEncoder.encode(word));
	sb.append("\">");
	sb.append(word);
	sb.append("</A>");
}
	public static String insertLinks(String phrase, String base) 
	{
		int idx = phrase.indexOf(LINK);
		if (idx == -1) return phrase;
		String token;
		StringTokenizer st;
		st = new StringTokenizer(phrase, " \n\t\r\'\"()[]$.,;:!?", true);
		StringBuffer sb = new StringBuffer();
		while (st.hasMoreTokens()) 
		{
			token = st.nextToken();
			if ((token.length() > 1) && (!token.equals("to"))) 
			{
				insertLink(token, base, sb);
			}
			else 
			{
				if (!token.equals(LINK)) sb.append(token);
			}
		}
		return sb.toString();
	}
	public Entry parseNextEntry() throws IOException 
	{

		Entry result = null;
		WordKind currentWordKind = new WordKind();
		Idiom currentIdiom = null;
		Meaning currentMeaning = null;
		MeaningContainer currentMeaningContainer = currentWordKind;
		String line, trimmedString;

		// Skip until next entry or end of input
		line = in.readLine();
		while ((line != null) && !line.startsWith(Entry.BEGIN)) 
		{
			//System.out.println(line+"-> null!"); 
			line = in.readLine();
		}

		if (line == null) 
		{ return null; }

		/* Now the variable line begins with @ and contains 
		the headword and its pronounciation */

		result = createEntry();
		result.parse(line);

		while ((line = in.readLine()) != null) 
		{
		    //System.out.println(line);
			if (line.length() == 0) break;
			if (line.trim().startsWith(WordKind.BEGIN)) 
			{
				trimmedString = line.trim().substring(WordKind.BEGIN.length()).trim();
				currentWordKind = createWordKind();
				currentWordKind.type = trimmedString;
				currentMeaningContainer = currentWordKind;
				result.addWordKind(currentWordKind);
			}
			else if (line.trim().startsWith(Idiom.BEGIN)) 
			{
				trimmedString = line.trim().substring(Idiom.BEGIN.length()).trim();
				currentIdiom = createIdiom(trimmedString);
				currentMeaningContainer = currentIdiom;
				if (currentWordKind != null) 
				{
					currentWordKind.addIdiom(currentIdiom);
				}
			}
			else if (line.trim().startsWith(Meaning.BEGIN)) 
			{
				trimmedString = line.trim().substring(Meaning.BEGIN.length()).trim();
				currentMeaning = createMeaning(trimmedString);
				if (currentMeaningContainer != null) 
				{
					currentMeaningContainer.addMeaning(currentMeaning);
				}
			}
			else if (line.trim().startsWith(Example.BEGIN)) 
			{
				trimmedString = line.trim().substring(Example.BEGIN.length()).trim();
				Example ex = createExample(null, null);
				ex.parse(trimmedString);
				currentMeaning.addExample(ex);
			}
		} // end while
		if (result.wordKinds.isEmpty()) {
		    currentWordKind.type = ".N.A.";
		    result.addWordKind(currentWordKind);
		}
		return result;
	}
public void setDataInput(DataInput anInputStream) {
	this.in = anInputStream;
}
}
