package vietdict.server;

import java.io.*;
import java.util.*;
import vietdict.common.LVCollator;

/**
 * Engine.java
 *
 *
 * Created: Tue May 16 17:23:32 2000
 *
 * @author 
 * @version
 */

public abstract class Engine  {
	
	int[] keys;

	private int binarySearch(byte[] key, int i, int j, RandomAccessDataSource f)
	throws Exception
	{
	if(i > j)
	    return -(i + 1);
	int k = (i + j) / 2;
	byte[] buff = readKey(k, f);
	int diff = compare(key, buff);
	if(diff == 0)
	    return k;
	if(diff < 0)
	    return binarySearch(key, i, k - 1, f);
	else
	    return binarySearch(key, k + 1, j, f);
	}
	public int binarySearch(byte[] key, RandomAccessDataSource f) throws Exception
	{
	return binarySearch(key, 0, keys.length - 1, f);
	}
	protected int compare(byte[] arr1, byte[] arr2) {
	//log(new String(arr1) + " - " + new String(arr2));
	return LVCollator.compare(arr1, arr2);
	}
	boolean eol(byte b) {
	return b == '\n' || b == '/';
	}
/**
 * Gets the structure where the data are stored
 */
protected RandomAccessDataSource getDataSource() throws java.io.IOException {
	return null;
}
	public byte[] getEntry(int pos, RandomAccessDataSource f) throws IOException {
	int len, start;
	start = keys[pos] >> 5;
	if (pos < keys.length-1) {
	    len = (keys[pos+1] >> 5) - start;
	}
	else {
	    len = (int) f.length() - start;
	}
	byte[] buff = new byte[len-1];
	f.seek(start);
	f.read(buff);
	return buff;
	}
	protected String[] getNeighbors(int pos, int number, RandomAccessDataSource f) 
	throws IOException
	{
	if (pos<0) pos = 0;
	String[] ret = new String[number];
	for (int i = 0; i < number; i++) {
	    ret[i] = new String(readKey(pos+i, f));
	}
	return ret;
	}
	protected int getPosition(byte[] key, RandomAccessDataSource f) 
	throws Exception 
	{
	int pos = binarySearch(key, f);
	if (pos < 0) {
	    pos = Math.min(-pos-1, keys.length-1);
	}
	return pos;
	}
public abstract String getReply(String s, int i, boolean t, boolean h, String b);
protected void initialize() throws IOException {
	RandomAccessDataSource in = getDataSource();
	byte[] buff = new byte[4096];
	int i = 0, k = 0, len = 0;
	Vector v = new Vector(10000);
	while ((len = in.read(buff)) != -1) {
		for (k = 0; k < len; k++) {
			if (buff[k] == '@') {
				v.addElement(new Integer(i + k));
			}
		}
		i = i + len;
	}
	in.close();
	keys = new int[v.size()];
	for (i = 0; i < keys.length; i++) {
		keys[i] = ((Integer) v.elementAt(i)).intValue();
	}
	v = null;
	RandomAccessDataSource f = getDataSource();
	buff = new byte[32];
	for (i = 0; i < keys.length; i++) {
		f.seek(keys[i] + 1);
		len = Math.min(32, f.read(buff));
		k = 0;
		while (k < len && !eol(buff[k])) {
			k++;
		}
		if (k == len)
			k = Math.min(len, 10);
		keys[i] = keys[i] * 32 + k;
	}
	f.close();
}
	public static void log(Object s) {
	System.out.println(s.toString());
	}
byte[] readKey(int k, RandomAccessDataSource f) throws IOException {
	int pos = keys[k] >> 5; // position of the key in file
	int len = keys[k] & 0x1f; // length of the key
	f.seek(pos + 1);
	byte[] buff = new byte[len];
	f.read(buff);
	return buff;
}
public void reorganize(String outFile) throws IOException {
	RandomAccessDataSource ds = getDataSource();
	com.sun.java.util.collections.Vector v = new com.sun.java.util.collections.Vector(keys.length);
	for (int i = 0; i < keys.length; i++){
		byte[] arr = readKey(i, ds);
		IndexedKey k = new IndexedKey();
		k.data = arr;
		k.index = i;
		v.addElement(k);
	}
	com.sun.java.util.collections.Collections.sort(v);
	OutputStream out = new FileOutputStream(outFile);
	for (int i = 0; i < v.size(); i++){
		IndexedKey k = (IndexedKey)v.elementAt(i);
		byte[] arr = getEntry(k.index, ds);
		out.write(arr);
		out.write('\n');
	}
	out.close();
	ds.close();
}
} // Engine
