package vietdict.server;

import java.net.*;
import java.util.*;
import java.io.*;
import vietdict.common.*;
import duc.util.*;
public class MultiDictEngine {
    Properties prop = new Properties();
	public static final int EV = 0;
	public static final int VE = 1;
	public static final int FV = 2;
	public static final int VF = 3;
	public static final int WN = 4;
	Engine[] engines = new Engine[5];
	boolean log;
    String cssLink;
	ByteArrayOutputStream logBuffer = new ByteArrayOutputStream();
	private static MultiDictEngine defaultInstance = null;
public MultiDictEngine(Properties prop) {
    this.prop = prop;
	String[] dataFiles = new String[5];
	log = Boolean.getBoolean(prop.getProperty("log","false"));
	cssLink = prop.getProperty("cssLink","http://www.informatik.uni-leipzig.de/~duc/Java/td.css");
	dataFiles[EV] = prop.getProperty("ev");
	dataFiles[VE] = prop.getProperty("ve");
	dataFiles[FV] = prop.getProperty("fv");
	dataFiles[VF] = prop.getProperty("vf");
	dataFiles[WN] = prop.getProperty("wn");
	for (int i = 0; i < 4; i++) {
		try {
			engines[i] = createFileEngine(dataFiles[i], i);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	try {
		engines[WN] = createZipEngine(dataFiles[WN]);
	} catch (Exception e) {
		e.printStackTrace();
	}
}
private Engine createFileEngine(String s, int id) throws IOException {
	if (s != null) {
		return new FileEngine(s, createParser(id));
	} else {
		return new DummyEngine();
	}
}
private Parser createParser(int i) {
	switch (i) {
		case FV: {
		    if ("true".equals(prop.getProperty("latin1"))) {
			return new FVParser();
		    } else {
			return new Parser();
		    }
		} 
		default: {
			return new Parser();
		}
	}
}
private Engine createZipEngine(String s) throws IOException {
	if (s != null) {
		return new ZipEngine(s);
	} else {
		return new DummyEngine();
	}
}
	public static MultiDictEngine getDefault()
	{
	return defaultInstance;
	}
public String getReply(int db, String s, int i, boolean tcvn, boolean html, String base) {
	log(db, s, tcvn);
	String s0 = toLowerCase(s.trim()).replace('_', ' ');
	if (db != EV && db != WN) {
		s0 = VIQREncoding.getInstance().toTCVN1(s0);
	}
	return engines[db].getReply(s0, i, tcvn, html, base);
}
	public String getReply(String progName, String fmt, int db, String s, int i)
	{
	boolean tcvn = false, html = true;
	if (fmt.indexOf('8') >= 0) tcvn = true;
	StringBuffer sb = new StringBuffer(20);
	if (db == WN) {
		sb.append(progName).append("?word=");
	} else {
		sb.append(progName).append("?db=").append(db).append("&fmt=").append(fmt).append("&word=");
	}
	if (db == FV) {
	    s = LV2Latin1.convertLAT2LV(s);
	}
		return getReply(db, s, i, tcvn, html, sb.toString());
	}
void log(int db, String word, boolean t) {
	if (!log) {
		return;
	}
	try {
		if (logBuffer.size() > 20480) {
			HNDWebServer.log(logBuffer.toString(), "" + System.currentTimeMillis() + ".txt");
			logBuffer.reset();
		}
		if (t) {
			logBuffer.write(("V" + db).getBytes());
		} else {
			logBuffer.write(("E" + db).getBytes());
		}
		logBuffer.write('>');
		logBuffer.write(word.getBytes());
		logBuffer.write('\n');
	} catch (IOException e) {
	}
}
	public static void setDefault(MultiDictEngine dictionary)
	{
	defaultInstance = dictionary;
	}
	private String toLowerCase(String s) {
	StringBuffer sb = new StringBuffer(s.length());
	char c;
	for (int i = 0; i < s.length(); i++) {
	    c = s.charAt(i);
	    if (c >= 'A' && c <= 'Z') {
		sb.append(Character.toLowerCase(c));
	    }
	    else {
		sb.append(c);
	    }
	}
	return sb.toString();
	}
}
