package vietdict.server;

import vietdict.netutil.*;
import java.net.*;
import java.io.*;
import java.util.*;

public class WebServerThread extends Thread 
{
	
	Socket theConnection;
	String docroot;
	
	static String indexfile = "index.html";
	
	protected String getError404()
	{
		return "<HTML><HEAD><TITLE>File Not Found</TITLE></HEAD>\n"+
				"<BODY><H1>HTTP Error 404: File Not Found</H1></BODY></HTML>";
	}
	protected String getError501()
	{
		return "<HTML><HEAD><TITLE>Not Implemented</TITLE></HEAD>"+
				"<BODY><H1>HTTP Error 501: Not Implemented</H1></BODY></HTML>";
	}
	protected String getVersion()
	{
		return "Ho Ngoc Duc's HTTP Server, 10/1999";
	}
	protected String guessContentTypeFromName(String name) 
	{
		String suffix = name.substring(1+name.lastIndexOf('.'));
		return NetToolkit.getContentTypeForName(suffix.toLowerCase());
	}
	public void initialize(Socket s, String dir) 
	{
		theConnection = s;
		docroot = dir;
	}
	public void run() 
	{
		String method = "";
		String file = "";
		String version = "";
		
		try 
		{
			PrintWriter os = new PrintWriter(theConnection.getOutputStream());
			BufferedReader is = 
				new BufferedReader(new InputStreamReader(theConnection.getInputStream()));
			String get = is.readLine();
			if (get == null) return;
			StringTokenizer st = new StringTokenizer(get);
			method = st.nextToken();
			if (method.equals("GET")) 
			{
				file = st.nextToken();
				if (file.endsWith("/")) file += indexfile;
				
				if (st.hasMoreTokens()) 
				{
					version = st.nextToken();
				}
				// loop through the rest of the input lines 
				while ((get = is.readLine()) != null) 
				{
					if (get.trim().equals("")) break;        
				}
				
				serveGET(os, file, version);
			}
			else 
			{  // method does not equal "GET"
				serve(os, method, version);
			}
			os.close();
		}
		catch (IOException e) // Ignore I/O exceptions 
		{			
		}
		
		// After the data have been sent, close the connection
		try 
		{
			theConnection.close();
		}
		catch (IOException e) 
		{
		}
	}
	protected void sendError404(PrintWriter os, String version)
	{
		String theString = getError404();
		if (version.startsWith("HTTP/"))
		{  // send a MIME header
			NetToolkit.printHTTPHeader(os,"404 File Not Found",getVersion(),
			theString.length(),"text/html");
		}
		os.println(theString);
	}
	protected void sendError501(PrintWriter os, String version)
	{
		String theString = getError501();
		if (version.startsWith("HTTP/")) 
		{  // send a MIME header
			NetToolkit.printHTTPHeader(os,"501 Not Implemented",getVersion(),theString.length(),"text/html");
		}       
		os.println(theString);	
	}
	protected void serve(PrintWriter os, String method, String version)
	{
		sendError501(os, version);
	}
	protected void serveGET(PrintWriter os, String file, String version)
	{
		if (file.indexOf("..") >= 0)
		{
			//System.err.println("Not serverd!");
			sendError404(os, version);
			os.close();
			return;
		}
		try 
		{
			File theFile = new File(docroot, file.substring(1,file.length()));
			BufferedReader fis = new BufferedReader(new FileReader(theFile));
			char[] theData = new char[(int) theFile.length()];
			// need to check the number of bytes read here
			fis.read(theData);
			fis.close();
			
			String ct = guessContentTypeFromName(file);
			if (version.startsWith("HTTP/")) 
			{  // send a MIME header
				NetToolkit.printHTTPHeader(os,"200 OK",getVersion(),theData.length,ct);
			}  // end try
			
			// send the file
			os.write(theData);
			os.close();
		}  // end try
		catch (IOException e) 
		{  // can't find the file
			sendError404(os, version);
			os.close();
		}
	}
}
